

--| Locals |--
local db
local started
local duration


--| GUI |--
local frame
local function build()
	frame = CreateFrame('Frame','NefGCD')
	frame:SetHeight(db.height)
	frame:SetWidth(db.width)
	frame:SetPoint(unpack(db.anchor))
	frame:SetResizable(true)
	frame:SetMovable(true)
	frame:SetParent(_G[db.parent])
	frame:SetScript('OnMouseDown',frame.StartMoving)
	frame:SetScript('OnMouseUp',function(self)
		self:StopMovingOrSizing()
		db.anchor = {self:GetPoint(1)}
	end)
	frame.Lock = function(self)
		self.guide:Hide()
		self.handle:Hide()
		self:EnableMouse(false)
		self.locked = true
	end
	frame.Unlock = function(self)
		self.guide:Show()
		self.handle:Show()
		self:EnableMouse(true)
		self.locked = false
	end
	frame.locked = true
	
	local bg = frame:CreateTexture(nil,'BACKGROUND')
	bg:SetAllPoints()
	bg:SetTexture(db.colorR,db.colorG,db.colorB,db.colorA)
	frame.bg = bg
	
	local spark = frame:CreateTexture(nil,'BORDER')
	spark:SetTexture('Interface\\AddOns\\NefGCD\\spark.tga')
	if db.orientation == 'HORIZONTAL' then
		spark:SetRotation(0)
		spark:SetHeight(db.height*3.2)
		spark:SetWidth(db.height*2.8)
	else
		spark:SetRotation(1.575)
		spark:SetHeight(db.width*2.8)
		spark:SetWidth(db.width*3.2)
	end
	spark:SetVertexColor(db.sparkR,db.sparkG,db.sparkB,db.sparkA)
	spark:Hide()
	frame.spark = spark
	
	local guide = frame:CreateTexture(nil,'OVERLAY')
	guide:SetTexture(0,1,0,.3)
	guide:SetAllPoints()
	guide:Hide()
	frame.guide = guide
	
	local handle = CreateFrame('Frame',nil,frame)
	handle:SetHeight(12)
	handle:SetWidth(12)
	handle:SetPoint('BOTTOMRIGHT')
	handle:EnableMouse(true)
	handle:SetScript('OnMouseDown',function() frame:StartSizing() end)
	handle:SetScript('OnMouseUp',function()
		frame:StopMovingOrSizing()
		db.height = frame:GetHeight()
		db.width = frame:GetWidth()
		if db.orientation == 'HORIZONTAL' then
			spark:SetHeight(db.height*3.2)  -- Update the spark.
			spark:SetWidth(db.height*2.8)
		else
			spark:SetHeight(db.width*2.8)
			spark:SetWidth(db.width*3.2)
		end
		db.anchor = {frame:GetPoint(1)}  -- Changing the size affects the anchor point. Resave it.
	end)
	handle:Hide()
	frame.handle = handle
	
	local handleImg = handle:CreateTexture(nil,'OVERLAY')
	handleImg:SetTexture('Interface\\AddOns\\NefGCD\\handle.tga')
	handleImg:SetAllPoints()
	handle.img = handleImg
	
	frame:SetFrameStrata(db.strata)
	frame:SetFrameLevel(db.level)  -- I think this needs to be the last action peformed on the frame. =/ 
	
	build = nil
end


--| Events |--
local e = CreateFrame('Frame')

e:RegisterEvent('ACTIONBAR_UPDATE_COOLDOWN')
e:RegisterEvent('VARIABLES_LOADED')
e:RegisterEvent('PLAYER_ENTERING_WORLD')

e:SetScript('OnEvent',function(self,event)
	if event == 'ACTIONBAR_UPDATE_COOLDOWN' then
		if not db.spell1 then return end
		local s1,d1 = GetSpellCooldown(db.spell1)  -- Try spell1. 
		if d1 > 0 and d1 < 2 then
			started = s1
			duration = d1
			frame.spark:Show()
			if db.bgType == 'DYNAMIC' then
				frame.bg:Show()
			end
			self:Show()
			return
		end
		if not db.spell2 then return end
		local s2,d2 = GetSpellCooldown(db.spell2)  -- Spell1 failed. School's probably locked. Try spell2.
		if d2 > 0 and d2 < 2 then
			started = s2
			duration = d2
			frame.spark:Show()
			if db.bgType == 'DYNAMIC' then
				frame.bg:Show()
			end
			self:Show()
			return
		end
	
	elseif event == 'VARIABLES_LOADED' then
		if not NefGCDDB then NefGCDDB = {} end
		setmetatable(NefGCDDB,{ __index = {
			height = 12, width = 200,  -- Defaults.
			strata = 'MEDIUM', level = 1,
			anchor = { 'CENTER',nil,'CENTER',0,0 },
			colorR = 0, colorG = 0, colorB = 0, colorA = .5,
			sparkR = 1, sparkG = 1, sparkB = 1, sparkA = 1,
			orientation = 'HORIZONTAL', reverse = false,
			parent = 'UIParent',
			bgType = 'CONSTANT',
		}})
		db = NefGCDDB
	
	elseif event == 'PLAYER_ENTERING_WORLD' then
		if build then build() end
		if not db.spell1 then
			print('|cff7fffffNefGCD:|r You must register a primary spell before NefGCD can work! Type |cffffff7f/gcd|r to register one..')
		end
		self:UnregisterEvent(event)  -- This event triggers at points during play also.
	
	end
end)


--| Animation |--
e:SetScript('OnUpdate',function(self)
	local perc = (GetTime() - started) / duration
	if perc > 1 then
		frame.spark:Hide()
		if db.bgType == 'DYNAMIC' then
			frame.bg:Hide()
		end
		self:Hide()
	else
		frame.spark:ClearAllPoints()
		if db.orientation == 'HORIZONTAL' then
			if db.reverse then
				frame.spark:SetPoint('CENTER',frame,'RIGHT',-(db.width*perc),0)
			else
				frame.spark:SetPoint('CENTER',frame,'LEFT',db.width*perc,0)
			end
		else
			if db.reverse then
				frame.spark:SetPoint('CENTER',frame,'TOP',0,-(db.height*perc))
			else
				frame.spark:SetPoint('CENTER',frame,'BOTTOM',0,db.height*perc)
			end
		end
	end
end)
e:Hide()


--| Config |--
local err1,err2
local opt = {
	name = 'NefGCD', type = 'group',
	args = {
		DESC = {
			name = 'A simple configurable GCD display.\n\n\n ', type = 'description', desc = '', order = 0,
		},
		LOCKED = {
			name = 'Locked', desc = 'Toggles locking the GCD frame (for movement and size).', type = 'toggle', order = 1,
			get = function()
				return frame.locked
			end,
			set = function()
				if frame.locked then
					frame:Unlock()
				else
					frame:Lock()
				end
			end,
		},
		PREVIEW = {
			name = 'Preview', desc = 'Starts a 20 second preview of the spark.', type = 'execute', order = 2,
			func = function()
				started = GetTime()
				duration = 20
				frame.spark:Show()
				if db.bgType == 'DYNAMIC' then
					frame.bg:Show()
				end
				e:Show()
			end,
		},
		BREAK1 = {
			name = ' \n\n ', type = 'description', desc = '', order = 3,
		},
		SPELL1 = {
			name = 'Primary Spell', desc = 'The primary spell to check for GCD activity.', type = 'input', order = 4,
			get = function() return db.spell1 or err1 or '' end,
			set = function(_,input)
				if GetSpellInfo(input) then
					err1 = nil
					db.spell1 = input
				else
					err1 = '|cffff7f7fError: Unknown spell!|r'
				end
			end,
		},
		SPELL2 = {
			name = 'Secondary Spell', desc = 'The secondary spell to check for GCD acitivity. Should be a different school from \'primary\'.', type = 'input', order = 5,
			get = function() return db.spell2 or err2 or '' end,
			set = function(_,input)
				if GetSpellInfo(input) then
					err2 = nil
					db.spell2 = input
				else
					err2 = '|cffff7f7fError: Unknown spell!|r'
				end
			end,
		},
		BREAK2 = {
			name = ' \n\n ', type = 'description', desc = '', order = 6,
		},
		STRATA = {
			name = 'Frame Strata', desc = 'The strata of the GCD frame.', type = 'select', order = 7,
			values = { FULLSCREEN = 'Fullscreen', HIGH = 'High', MEDIUM = 'Medium', LOW = 'Low', BACKGROUND = 'Background' },
			get = function() return db.strata end,
			set = function(_,input)
				db.strata = input
				frame:SetFrameStrata(input)
				frame:SetFrameLevel(db.level)  -- Re-set the Z-ordering.
			end,
		},
		LEVEL = {
			name = 'Frame Level', desc = 'The frame level (z-order) of the GCD frame.', type = 'range', order = 8,
			min = 0, max = 100, step = 1,
			get = function() return db.level end,
			set = function(_,input)
				db.level = input
				frame:SetFrameStrata(db.strata)  -- Re-set the Z-ordering.
				frame:SetFrameLevel(input)
			end,
		},
		BREAK3 = {
			name = ' ', type = 'description', desc = '', order = 9,
		},
		PARENT = {
			name = 'Frame Parent', desc = 'The frame to parent the GCD indicator to. WARNING: Must exist at the PLAYER_ENTERING_WORLD event.', type = 'input', order = 10,
			get = function() return db.parent end,
			set = function(_,input)
				if type(_G[input]) == 'table' and _G[input].GetObjectType then
					db.parent = input
					frame:SetParent(_G[input])
					frame:SetFrameStrata(db.strata)  -- Re-set Z-order for the new parent.
					frame:SetFrameLevel(db.level)
				else
					print('|cff7fffffNefGM:|r Unknown frame \''..input..'\'.')
				end
			end,
		},
		BREAK4 = {
			name = ' \n\n ', type = 'description', desc = '', order = 11,
		},
		ORIENTATION = {
			name = 'Orientation', desc = 'The orientation for the GCD indicator.', type = 'select', order = 12,
			values = { VERTICAL = 'Vertical', HORIZONTAL = 'Horizontal' },
			get = function() return db.orientation end,
			set = function(_,input)
				if input == 'HORIZONTAL' then
					frame.spark:SetRotation(0)
					frame.spark:SetHeight(db.height*3.2)
					frame.spark:SetWidth(db.height*2.8)
				else
					frame.spark:SetRotation(1.575)
					frame.spark:SetHeight(db.width*2.8)
					frame.spark:SetWidth(db.width*3.2)
				end
				db.orientation = input
			end,
		},
		REVERSE = {
			name = 'Reverse', desc = 'Reverses the direction of the GCD spark.', type = 'toggle', order = 13,
			get = function() return db.reverse end,
			set = function() db.reverse = not db.reverse end,
		},
		BREAK5 = {
			name = ' \n\n ', type = 'description', desc = '', order = 14,
		},
		COLOR = {
			name = 'BG Color', desc = 'The color of the GCD frame backdrop.', type = 'color', order = 15,
			hasAlpha = true,
			get = function() return db.colorR,db.colorG,db.colorB,db.colorA end,
			set = function(_,r,g,b,a)
				db.colorR,db.colorG,db.colorB,db.colorA = r,g,b,a
				frame.bg:SetTexture(r,g,b,a)
			end,
		},
		BG_TYPE = {
			name = 'BG Type', type = 'select', desc = 'If constant, the background will always be visible. If dynamic, it will appear only when the spark is active.', order = 15,
			values = { CONSTANT = 'Constant', DYNAMIC = 'Dynamic' },
			get = function() return db.bgType end,
			set = function(_,input)
				if input == 'CONSTANT' then
					frame.bg:Show()
				else
					if not frame.spark:IsShown() then frame.bg:Hide() end
				end
				db.bgType = input
			end,
		},
		BREAK6 = {
			name = ' \n\n ', type = 'description', desc = '', order = 17,
		},
		SPARK_COLOR = {
			name = 'Spark Color', desc = 'The color of the spark.', type = 'color', order = 18,
			hasAlpha = true,
			get = function() return db.sparkR,db.sparkG,db.sparkB,db.sparkA end,
			set = function(_,r,g,b,a)
				db.sparkR,db.sparkG,db.sparkB,db.sparkA = r,g,b,a
				frame.spark:SetVertexColor(r,g,b,a)
			end,
		},
	},
}

LibStub('AceConfig-3.0'):RegisterOptionsTable('NefGCD',opt)
LibStub('AceConfigDialog-3.0'):AddToBlizOptions('NefGCD')

SLASH_NEFGCD1 = '/nefgcd'
SLASH_NEFGCD2 = '/gcd'
SlashCmdList.NEFGCD = function()
	InterfaceOptionsFrame_OpenToCategory('NefGCD')
end
